#include <cstdarg>
#include <cstdio>

#include "utilities.h"
#include "bufferedtextfile.h"



BufferedTextFile::BufferedTextFile()
:
    mFile(0),
    mBuffer(0),
    mBufferSize(0),
    mBufferCapacity(0)
{}


BufferedTextFile::BufferedTextFile(const Filename& filePath,
                                   FILEOPEN        mode,
                                   FILEDIALOG      errDialog)
:   mFile(0),
    mBuffer(0),
    mBufferSize(0),
    mBufferCapacity(0)
{
    open(filePath, mode, errDialog);
}


BufferedTextFile::~BufferedTextFile()
{
    close();
}


bool BufferedTextFile::open(const Filename& path,
                            FILEOPEN        mode,
                            FILEDIALOG      errDialog)
{
    if (isOpen()) { close(); }

    mBuffer = bNew CHAR[524288];
    if (!mBuffer)
    {
        ERRLOG_RETURN_VALUE(false, "BufferedTextFile::open(): Could not allocate buffer");
    }

    if ((mFile = BaseFile::Alloc()) == NULL)
    {
        bDelete(mBuffer);
        ERRLOG_RETURN_VALUE(false, "BufferedTextFile::open(): Could not allocate BaseFile");
    }

    if (!mFile->Open(path, mode, errDialog))
    {
        BaseFile::Free(mFile);
        bDelete(mBuffer);
        ERRLOG_RETURN_VALUE(false, "BufferedTextFile::open(): Could not open file '" + path.GetString() + "'");
    }

    mBufferSize     = 0;
    mBufferCapacity = 524288;
    return true;
}


void BufferedTextFile::close()
{
    if (isOpen())
    {
        flush();
        BaseFile::Free(mFile);
        bDelete(mBuffer);
        mBufferSize     = 0;
        mBufferCapacity = 0;
    }
}


bool BufferedTextFile::write(LONG v)
{
    static CHAR sBuffer[32];

    sprintf(sBuffer, "%d", (int)v);
    return write(sBuffer);
}


bool BufferedTextFile::write(ULONG v)
{
    static CHAR sBuffer[32];

    sprintf(sBuffer, "%u", (unsigned int)v);
    return write(sBuffer);
}


bool BufferedTextFile::write(SReal v)
{
    static CHAR sBuffer[32];

    sprintf(sBuffer, "%.8g", v);
    return write(sBuffer);
}


bool BufferedTextFile::write(LReal v)
{
    static CHAR sBuffer[32];

    sprintf(sBuffer, "%.8g", v);
    return write(sBuffer);
}


bool BufferedTextFile::write(const Vector& v)
{
    static CHAR sBuffer[128];

    sprintf(sBuffer, "%.8g %.8g %.8g", v.x, v.y, v.z);
    return write(sBuffer);
}


bool BufferedTextFile::write(const String& s)
{
    static CHAR sBuffer[4096];

    s.GetCString(sBuffer, sizeof(sBuffer), STRINGENCODING_UTF8);
    sBuffer[sizeof(sBuffer)-1] = '\0';
    return write(sBuffer);
}


bool BufferedTextFile::write(char v)
{
    if (!isOpen()) { return false; }

    if (mBufferSize == mBufferCapacity)
    {
        if (!flush()) { return false; }
    }
    mBuffer[mBufferSize] = v;
    ++mBufferSize;
    return true;
}


bool BufferedTextFile::write(const char* s)
{
    if (!isOpen()) { return false; }

    while (*s != '\0')
    {
        if (mBufferSize == mBufferCapacity)
        {
            if (!flush()) { return false; }
        }
        mBuffer[mBufferSize] = *s;
        ++s;
        ++mBufferSize;
    }
    return true;
}


bool BufferedTextFile::flush()
{
    if (!isOpen()) { return false; }

    bool retVal = true;
    if (mBufferSize)
    {
        retVal = mFile->WriteBytes(mBuffer, mBufferSize);
        mBufferSize = 0;
    }
    return retVal;
}
