#ifndef C4D_ALLOCATOR_H
#define C4D_ALLOCATOR_H 1



#include <c4d.h>
#include <c4d_memory.h>
#include <limits>
#include <new>



/// Allocator that can be used in STL containers to make them use the C4D memory
/// functions.
template <typename T>
class C4DAllocator
{
public:

    typedef T           value_type;
    typedef size_t      size_type;
    typedef ptrdiff_t   difference_type;
    typedef T*          pointer;
    typedef const T*    const_pointer;
    typedef T&          reference;
    typedef const T&    const_reference;

    template <typename O>  struct rebind
    {
        typedef C4DAllocator<O> other;
    };


    inline C4DAllocator();

    inline C4DAllocator(const C4DAllocator& other);

    template <typename U>
    inline C4DAllocator(const C4DAllocator<U>& other);

    inline T* address(T& v) const;

    inline const T* address(const T& v) const;

    inline size_t max_size() const;

    inline T* allocate(const size_t size,
                       const void*  hint = NULL) const;

    inline void deallocate(T*     p,
                           size_t size) const;

    inline void construct(T*       p,
                          const T& value) const;

    inline void destroy(T* p) const;
};


template <typename T>
inline bool operator==(const C4DAllocator<T>& allocator1,
                       const C4DAllocator<T>& allocator2)
{
    return true;
}


template <typename T>
inline bool operator!=(const C4DAllocator<T>& allocator1,
                       const C4DAllocator<T>& allocator2)
{
    return false;
}



//-----------------------------------------------------------------------------
// Implementation(s) for C4DAllocator


template <typename T>
inline C4DAllocator<T>::C4DAllocator()
{}


template <typename T>
inline C4DAllocator<T>::C4DAllocator(const C4DAllocator& other)
{}


template <typename T>
template <typename U>
inline C4DAllocator<T>::C4DAllocator(const C4DAllocator<U>& other)
{}


template <typename T>
inline T* C4DAllocator<T>::address(T& v) const
{
    return &v;
}


template <typename T>
inline const T* C4DAllocator<T>::address(const T& v) const
{
    return &v;
}


template <typename T>
inline size_t C4DAllocator<T>::max_size() const
{
    return std::numeric_limits<LONG>::max() / sizeof(T);
}


template <typename T>
inline T* C4DAllocator<T>::allocate(const size_t size,
                                    const void*  hint) const
{
    return (T*)GeAllocNC((LONG)(size * sizeof(T)));
}


template <typename T>
inline void C4DAllocator<T>::deallocate(T*     p,
                                        size_t size) const
{
    GeFree(p);
}


template <typename T>
inline void C4DAllocator<T>::construct(T*       p,
                                       const T& value) const
{
    ::new(p) T(value);
}


template <typename T>
inline void C4DAllocator<T>::destroy(T* p) const
{
    p->~T();
}



#endif  // #ifndef C4D_ALLOCATOR_H
