#ifndef _COMMON_H_
#define _COMMON_H_ 1



#include <c4d.h>
#include <customgui_datetime.h>



//-----------------------------------------------------------------------------
// Common types


typedef VULONG SizeT;



//-----------------------------------------------------------------------------
// Backwards compatibility



#if _C4D_VERSION < 110

#define GE_FCOPY_OVERWRITE      TRUE

#endif



#if _C4D_VERSION < 115

#define MINLONGl  MINLONG
#define MAXLONGl  MAXLONG

#endif



#if _C4D_VERSION < 120


class AutoBitmap
{
public:

    AutoBitmap(const String& name);
    ~AutoBitmap();

    operator BaseBitmap* () const { return mBitmap; }


private:

    BaseBitmap *mBitmap;
};

enum COPYFLAGS
{
    COPYFLAGS_0                  = 0,
    COPYFLAGS_NO_HIERARCHY       = COPY_NO_HIERARCHY,
    COPYFLAGS_NO_ANIMATION       = COPY_NO_ANIMATION,
    COPYFLAGS_NO_BITS            = COPY_NO_BITS,
    //COPYFLAGS_NO_MATERIALPREVIEW = (1<<5),
    COPYFLAGS_NO_BRANCHES        = COPY_NO_BRANCHES,
    COPYFLAGS_DOCUMENT           = COPY_DOCUMENT,
    COPYFLAGS_NO_NGONS           = COPY_NONGONS,
    //COPYFLAGS_CACHE_BUILD        = (1<<13),
#if _C4D_VERSION >= 115
    COPYFLAGS_RECURSIONCHECK     = COPY_RECURSIONCHECK
#endif
};

#define COPYFLAGS   LONG

typedef tagDateTime DateTime;
#define GetDateTimeNow(dateTime)  DateTimeNow(dateTime)

enum DESCFLAGS_GET
{
    DESCFLAGS_GET_0         = 0,
    DESCFLAGS_GET_PARAM_GET = DESCFLAGS_PARAM_GET,
};

enum DESCFLAGS_SET
{
    DESCFLAGS_SET_0                 = 0,
    DESCFLAGS_SET_SET               = DESCFLAGS_PARAM_SET,
    DESCFLAGS_SET_USERINTERACTION   = DESCFLAGS_USERINTERACTION,
    DESCFLAGS_SET_DONTCHECKMINMAX   = DESCFLAGS_DONTCHECKMINMAX
};


enum DLG_TYPE
{
    DLG_TYPE_MODAL                  = FALSE,
    DLG_TYPE_MODAL_RESIZEABLE       = FALSE,

    DLG_TYPE_ASYNC                  = TRUE,
    DLG_TYPE_ASYNC_POPUP_RESIZEABLE = TRUE,
    DLG_TYPE_ASYNC_POPUPEDIT        = TRUE
};

#define EXECUTIONPRIORITY_INITIAL         EXECUTION_INITIAL
#define EXECUTIONPRIORITY_ANIMATION       EXECUTION_ANIMATION
#define EXECUTIONPRIORITY_ANIMATION_NLA   EXECUTION_ANIMATION
#define EXECUTIONPRIORITY_EXPRESSION      EXECUTION_EXPRESSION
#define EXECUTIONPRIORITY_DYNAMICS        EXECUTION_DYNAMICS
#define EXECUTIONPRIORITY_GENERATOR       EXECUTION_GENERATOR

enum FILEDIALOG
{
    FILEDIALOG_NONE       = FILE_NODIALOG,
    FILEDIALOG_ANY        = FILE_DIALOG,
    FILEDIALOG_IGNOREOPEN = FILE_IGNOREOPEN
};

enum FILEOPEN
{
    FILEOPEN_APPEND       = GE_APPEND,
    FILEOPEN_READ         = GE_READ,
    FILEOPEN_WRITE        = GE_WRITE,
    FILEOPEN_READWRITE    = GE_READWRITE,
    FILEOPEN_READ_NOCACHE = GE_READ,
    FILEOPEN_SHAREDREAD   = GE_READ
};

enum FILESELECT
{
    FILESELECT_LOAD       = 0,
    FILESELECT_SAVE       = GE_SAVE,
    FILESELECT_DIRECTORY  = GE_DIRECTORY
};

enum FILESELECTTYPE
{
    FILESELECTTYPE_ANYTHING   = FSTYPE_ANYTHING,
    FILESELECTTYPE_IMAGES     = FSTYPE_IMAGES,
    FILESELECTTYPE_SCENES     = FSTYPE_SCENES,
    FILESELECTTYPE_COFFEE     = FSTYPE_COFFEE,
    //FILESELECTTYPE_BODYPAINT  = 4
};

#define FreeChildren            FreeChilds

enum IMAGERESULT
{
    IMAGERESULT_OK            = IMAGE_OK,
    IMAGERESULT_NOTEXISTING   = IMAGE_NOTEXISTING,
    IMAGERESULT_WRONGTYPE     = IMAGE_WRONGTYPE,
    IMAGERESULT_OUTOFMEMORY   = IMAGE_NOMEM,
    IMAGERESULT_FILEERROR     = IMAGE_DISKERROR,
    IMAGERESULT_FILESTRUCTURE = IMAGE_FILESTRUCT,
    //IMAGERESULT_MISC_ERROR    = -6,
    //IMAGERESULT_PARAM_ERROR   = -7
};

#define MINREALr  MINREAL
#define MAXREALr  MAXREAL

#define STRINGENCODING_XBIT     StXbit
#define STRINGENCODING_8BIT     St8bit
#define STRINGENCODING_7BIT     St7bit
#define STRINGENCODING_7BITHEX  St7bithex
#define STRINGENCODING_UTF8     StUTF8
#define STRINGENCODING_HTML     StHTML

typedef const void* UVWHandle;

typedef Real   SReal;
typedef Vector SVector;
typedef Matrix SMatrix;

inline SVector toSV(const SVector& other)  { return other; }
inline SVector toSV(const LVector& other)  { return SV(other); }
inline LVector toLV(const SVector& other)  { return LV(other); }
inline LVector toLV(const LVector& other)  { return other; }
inline Vector  toV (const SVector& other)  { return other; }
inline Vector  toV (const LVector& other)  { return SV(other); }

inline SMatrix toSM(const SMatrix& other)  { return other; }
inline SMatrix toSM(const LMatrix& other)  { return SM(other); }
inline LMatrix toLM(const SMatrix& other)  { return LM(other); }
inline LMatrix toLM(const LMatrix& other)  { return other; }
inline Matrix  toM (const SMatrix& other)  { return other; }
inline Matrix  toM (const LMatrix& other)  { return SM(other); }

#else

inline SVector toSV(const SVector& other)  { return other; }
inline SVector toSV(const LVector& other)  { return other.ToSV(); }
inline LVector toLV(const SVector& other)  { return other.ToLV(); }
inline LVector toLV(const LVector& other)  { return other; }
inline Vector  toV (const SVector& other)  { return other.ToLV(); }
inline Vector  toV (const LVector& other)  { return other; }

inline SMatrix toSM(const SMatrix& other)  { return other; }
inline SMatrix toSM(const LMatrix& other)  { return other.ToSM(); }
inline LMatrix toLM(const SMatrix& other)  { return other.ToLM(); }
inline LMatrix toLM(const LMatrix& other)  { return other; }
inline Matrix  toM (const SMatrix& other)  { return other.ToLM(); }
inline Matrix  toM (const LMatrix& other)  { return other; }

#endif


inline bool fileSelect(Filename&      path,
                       FILESELECTTYPE type,
                       FILESELECT     flags,
                       const String&  title)
{
#if _C4D_VERSION >= 120
    return path.FileSelect(type, flags, title);
#else
    return path.FileSelect(type, flags, &title);
#endif
}


#endif
