#ifndef _DLIST_H_
#define _DLIST_H_  1



#include <c4d.h>

#include "utilities.h"



template <class T> class DListIterator;
template <class T> class ConstDListIterator;



///
template<class T>
class DList
{
    friend class ConstDListIterator<T>;
    friend class DListIterator<T>;


public:

    typedef ConstDListIterator<T> ConstIteratorT;
    typedef DListIterator<T>      IteratorT;


    DList();
    ~DList();

    DList(const DList& other);
    DList& operator=(const DList& other);

    bool appendFront();
    bool appendBack();
    void removeFront();
    void removeBack();

    bool pushFront(const T& value);
    bool pushBack(const T& value);

    bool pushFront(ConstIteratorT& start,
                   ConstIteratorT& stop);
    bool pushBack(ConstIteratorT& start,
                  ConstIteratorT& stop);

    bool pushFront(const DList& other);
    bool pushBack(const DList& other);

    T popFront();
    T popBack();

    void remove(IteratorT& iter);

    void adopt(IteratorT& entry);
    void adopt(IteratorT& entry,
               IteratorT& at);

    void adoptFront(DList& other);
    void adoptBack(DList& other);

    inline SizeT size() const;

    void erase();

    inline const T& front() const;
    inline T&       front();
    inline const T& back() const;
    inline T&       back();

    inline ConstIteratorT begin() const;
    inline IteratorT      begin();
    inline ConstIteratorT end() const;
    inline IteratorT      end();


private:

    struct Node
    {
        Node* mPrevious;
        Node* mNext;
        T     mData;

        Node(Node* previous,
             Node* next)
        : 
            mPrevious(previous),
            mNext(next)
        {}

        Node(Node*    previous,
             Node*    next,
             const T& value)
        :
            mPrevious(previous),
            mNext(next),
            mData(value)
        {}
    };


    Node* mFront;
    Node* mBack;
    SizeT mSize;
};



///
template <class T>
class DListIterator
{
    friend class DList<T>;
    friend class ConstDListIterator<T>;


public:

    typedef DList<T> ContainerT;


    DListIterator();

    inline bool isValid() const;
    inline bool isFront() const;
    inline bool isBack() const;

    inline ContainerT* container();

    inline T& operator*() const;
    inline T* operator->() const;
    inline DListIterator& operator++();
    inline DListIterator& operator--();

    inline bool operator==(const ConstDListIterator<T>& other) const;
    inline bool operator==(const DListIterator<T>& other) const;
    inline bool operator!=(const ConstDListIterator<T>& other) const;
    inline bool operator!=(const DListIterator<T>& other) const;


private:

    typedef typename ContainerT::Node  NodeT;

    ContainerT* mList;
    NodeT*      mNode;


    DListIterator(ContainerT& list,
                  NodeT*      node);
};



///
template <class T>
class ConstDListIterator
{
    friend class DList<T>;
    friend class DListIterator<T>;


public:

    typedef DList<T> ContainerT;


    ConstDListIterator();

    ConstDListIterator(const DListIterator<T>& other);
    ConstDListIterator& operator=(const DListIterator<T>& other);

    inline bool isValid() const;
    inline bool isFront() const;
    inline bool isBack() const;

    inline const ContainerT* container() const;

    inline const T& operator*() const;
    inline const T* operator->() const;
    inline ConstDListIterator& operator++();
    inline ConstDListIterator& operator--();

    inline bool operator==(const ConstDListIterator<T>& other) const;
    inline bool operator==(const DListIterator<T>& other) const;
    inline bool operator!=(const ConstDListIterator<T>& other) const;
    inline bool operator!=(const DListIterator<T>& other) const;


private:

    typedef typename ContainerT::Node  NodeT;

    const ContainerT* mList;
    const NodeT*      mNode;


    ConstDListIterator(const ContainerT& list,
                       const NodeT*      node);
};

#include "DList_impl.h"



#endif  // #ifndef _DLIST_H_
