#ifndef _FILEPATH_H_
#define _FILEPATH_H_  1



#include <c4d.h>

#include "dlist.h"



///
class FilePath
{
public:

    FilePath();
    FilePath(const Filename& filename);
    FilePath(const String& pathStr);

    FilePath(const FilePath& other);
    FilePath& operator=(const FilePath& other);

    FilePath& operator=(const Filename& filename);
    FilePath& operator=(const String& pathStr);

    void clear();
    inline bool isEmpty() const;
    inline bool isAbsolute() const;

    FilePath getDirectoryPath() const;

    String getString() const;
    String getFileString() const;
    String getDirectoryString() const;

    Filename getFilename() const;

    void makeRelativeTo(const FilePath& basePath);

    FilePath operator+(const FilePath& other) const;

    FilePath& operator+=(const String& token);

    /// Returns the suffix of the last token (i.e. the file).
    String getSuffix() const;

    /// Sets the suffix of the last token (i.e. the file).
    void setSuffix(const String& suffix);

    /// Special function that replaces the sequence index of a file with
    /// another index. The new sequence will have the same number of digits
    /// as the old one.
    ///
    /// @param[in]  newIndex
    ///     The new index that will replace the old one. If the new index is
    ///     negative, the index will be replaced by "xxx...".
    void replaceIndex(const LONG newIndex);

    /// Appends some text to the file name of the last token, but makes sure
    /// that it is inserted in front of the suffix (if the last token has one).
    void appendToFile(const String& text);




private:

    typedef DList<String> TokensT;

    CHAR    mDrive;
    bool    mIsAbsolute;
    TokensT mTokens;
};



//-----------------------------------------------------------------------------
// Inlined functions of FilePath


inline bool FilePath::isEmpty() const
{
    return (!mDrive && !mIsAbsolute && !mTokens.size());
}


inline bool FilePath::isAbsolute() const
{
    return mIsAbsolute;
}



#endif  // #ifndef _FILEPATH_H_
