#ifndef _OE_CONVERTER_H_
#define _OE_CONVERTER_H_  1



#include <c4d.h>

#include "autoref.h"
#include "common.h"
#include "filepath.h"
#include "oe_obj_converter.h"
#include "stl_containers.h"
#include "utilities.h"



struct OEConverter
{
    // project configuration
    BaseDocument*       mDocument;
    Filename            mProjectPath;
    String              mProjectName;
    StdVector<String>   mRenderTargets;
    ULONG               mRenderTargetIx;
    LONG                mExportScaleId;
    bool                mCollectAllImages;
    bool                mUseDocumentScale;
    ULONG               mSamples;
    Filename            mOutputFilename;
    LONG                mImageFormatId;

    // export configuration
    bool                mExportResolution;
    ULONG               mWidth;
    ULONG               mHeight;
    LONG                mResolutionScaleId;
    bool                mExportCamera;
    ULONG               mCameraIx;
    bool                mExportAperture;
    Real                mAperture;
    bool                mExportFocalDepth;
    Real                mFocalDepth;
    bool                mExportMotionBlur;
    bool                mMotionBlurToNext;
    bool                mExportSunDirection;
    ULONG               mLightIx;


    OEConverter();

    void clear();

    CameraObject* getCamera();

    Real getFocalDepth(
        CameraObject &cam) const;

    Real getExportScale() const;

    bool exportToObj(
        const bool isPreExport);

    bool launchOctane(
        const bool   isAnim,
        const LONG   frame,
        const Matrix *mbCamMat1 = 0,
        const Matrix *mbCamMat2 = 0);

    bool relaunchOctane();

    bool stopOctane();

    bool octaneIsActive();

    bool imageRendered();


private:

    // set by exportToObj()
    bool                        mWasPreExport;
    OEOBJConverter::RenameListT mTempRenames;
    FilePath                    mObjFilePath;
    FilePath                    mOcsFilePath;
    Vector                      mSunDir;
    // set by launchOctane()
    String                      mArgs[200];
    SizeT                       mArgCount;
    FilePath                    mOutputFilePath;
    FilePath                    mOctaneFilePath;
    bool                        mIsAnim;
    ProcessHandle               mOctaneHandle;


    bool renameTemps();

    static void calcOctaneCamera(
        const Matrix &matrix,
        const Real   focalDepth,
        const Real   scale,
        const bool   animation,
        Vector       &pos,
        Vector       &target,
        Vector       &up);

    static String toString(
        const Real value);

    OEConverter(const OEConverter& other);
    OEConverter& operator=(const OEConverter& other);
};

typedef AutoRef<OEConverter>  OEConverterH;



#endif  // #ifndef _OE_CONVERTER_H_
