#ifndef _OE_EXPORT_DLG_H_
#define _OE_EXPORT_DLG_H_  1



#include <c4d.h>

#include "utilities.h"



class OESceneHook;
struct OEConverter;



///
class OEExportDlg : public GeDialog
{
    INSTANCEOF(OEExportDlg, GeDialog);

public:

    static const Real cExportScales[ID_OE_DLG_NATIVE_UNIT_NUMBER];
    static const Real cResolutionScales[ID_OE_DLG_SCALE_NUMBER];


    static bool readExportSettings(
        BaseDocument &document,
        OEConverter  &converter);

    static inline Real getExportScale(
        const LONG id);

    static inline Real getResolutionScale(
        const LONG id);


    OEExportDlg();

    virtual ~OEExportDlg();

    void run();


    // --- Implementation(s) for GeDialog ---

    virtual Bool CreateLayout();

    virtual Bool InitValues();

    virtual Bool CoreMessage(
        LONG                id,
        const BaseContainer &msg);

    virtual Bool Command(
        LONG                id,
        const BaseContainer &msg);

    virtual Bool AskClose();

    virtual void Timer(
        const BaseContainer &msg);

    virtual void DestroyWindow();


private:

    static const LONG   cTimerInterval;
    static const ULONG  cMaxRetries;
    static const LONG   cExportDelay;
    static const LONG   cMaxRetryDelay;
    static const LONG   cNextFrameDelay;

    // members for export dialog
    BaseDocument   *mDocument;
    BaseContainer  *mSettings;
    BaseContainer  *mRenderTargetSettings;
    AtomArray      *mCameras;
    AtomArray      *mLights;

    // members for animation export
    OEConverter    *mAnimConverter;
    LONG           mStartFrame;
    LONG           mEndFrame;
    LONG           mCurrentFrame;
    ULONG          mStep;
    bool           mOnlyCameraAnimation;
    bool           mCameraAnimationSceneExported;
    ULONG          mFailureCount;
    LONG           mTimeSinceOctaneLaunch;
    LONG           mTimeSinceOctaneExit;
    LONG           mExportedFrame;
    bool           mCalledClose;
    Matrix         mMBCamMat1;
    Matrix         mMBCamMat2;
    bool           mMBCamMatsValid;


    void copyBoolFrom(
        const LONG          id,
        const BaseContainer *container,
        const bool          preset = false);

    void copyBoolTo(
        const LONG    id,
        BaseContainer *container);


    void copyLongFrom(
        const LONG          id,
        const BaseContainer *container,
        const LONG          preset = 0,
        const LONG          min = MINLONGl,
        const LONG          max = MAXLONGl,
        const LONG          step = 1);

    void copyLongTo(
        const LONG    id,
        BaseContainer *container);


    void copyRealFrom(
        const LONG          id,
        const BaseContainer *container,
        const Real          preset = 0.0,
        const Real          min = MINREALr,
        const Real          max = MAXREALr,
        const Real          min2 = 0.0,
        const Real          max2 = 0.0,
        const Real          step = 1.0,
        const LONG          format = FORMAT_REAL);

    void copyRealTo(
        const LONG    id,
        BaseContainer *container);


    void copyStringFrom(
        const LONG          id,
        const BaseContainer *container,
        const String        &preset = String());

    void copyStringTo(
        const LONG    id,
        BaseContainer *container);


    bool getBool(
        const LONG id,
        const bool preset = false);

    LONG getLong(
        const LONG id,
        const LONG preset = 0);

    Real getReal(
        const LONG id,
        const Real preset = 0.0);

    String getString(
        const LONG   id,
        const String preset = String());


    static void cleanupFilename(
        String &str);

    void cleanupFilename(
        const LONG id);


    void toggleRenderTargetButtons();

    void toggleNativeUnit();

    void toggleFilm();

    void toggleCamera();

    void toggleLight();

    void togglePixelSamples();


    String toString(
        const Real value);


    static BaseContainer* getRenderTargetSettings(
        BaseContainer &settings);

    void addRenderTarget();

    void renameRenderTarget();

    void removeRenderTarget();


    void exportAnimation();

    void stopAnimationExport();

    void setMessage(
        const String& message);

    bool exportCurrentFrame();

    bool exportCurrentState();

    bool renderCurrentFrame();
};



inline Real OEExportDlg::getExportScale(
    const LONG id)
{
    GeAssert((id >= 0) && (id < ID_OE_DLG_NATIVE_UNIT_NUMBER));
    return cExportScales[id];
}


inline Real OEExportDlg::getResolutionScale(
    const LONG id)
{
    GeAssert((id >= 0) && (id < ID_OE_DLG_SCALE_NUMBER));
    return cResolutionScales[id];
}



#endif  // #ifndef _OE_EXPORT_DLG_H_
