#include <c4d.h>

#include "oe_about.h"
#include "oe_export.h"
#include "oe_instance_tag.h"
#include "oe_preferences.h"
#include "oe_render.h"
#include "oe_scenehook.h"
#include "utilities.h"



/// Hook that is called during the launch of CINEMA 4D, when the DLL
/// was loaded and the plugins should be initialised and registered.
///
/// @return
///   true if successful, otherwise false.
Bool PluginStart()
{
    // initialize global resource object
    if (!resource.Init())
    {
        ERRLOG_RETURN_VALUE(FALSE, "Global resource object could not be initialized.");
    }

    // register OESceneHook
    if (!OESceneHook::registerPlugin())
    {
        ERRLOG_RETURN_VALUE(FALSE, "Could not register OESceneHook plugin.");
    }

    // register OEExport
    OEExport* oeExport = gNew OEExport;
    if (!oeExport)
    {
        ERRLOG_RETURN_VALUE(FALSE, "Could not allocate OEExport plugin.");
    }
    if (!oeExport->registerPlugin())
    {
        gDelete(oeExport);
        ERRLOG_RETURN_VALUE(FALSE, "Could not register OEExport plugin.");
    }

    // register OERender
    OERender* oeRender = gNew OERender;
    if (!oeRender)
    {
        ERRLOG_RETURN_VALUE(FALSE, "Could not allocate OERender plugin.");
    }
    if (!oeRender->registerPlugin())
    {
        gDelete(oeRender);
        ERRLOG_RETURN_VALUE(FALSE, "Could not register OERender plugin.");
    }

    // register OEPreferences
    gPreferences = gNew OEPreferences;
    if (!gPreferences)
    {
        ERRLOG_RETURN_VALUE(FALSE, "Could not allocate OEPreferences plugin.");
    }
    if (!gPreferences->registerPlugin())
    {
        gDelete(gPreferences);
        ERRLOG_RETURN_VALUE(FALSE, "Could not register OEPreferences plugin.");
    }

    // register OEAbout
    OEAbout* oeAbout = gNew OEAbout;
    if (!oeAbout)
    {
        ERRLOG_RETURN_VALUE(FALSE, "Could not allocate OEAbout plugin.");
    }
    if (!oeAbout->registerPlugin())
    {
        gDelete(oeAbout);
        ERRLOG_RETURN_VALUE(FALSE, "Could not register OEAbout plugin.");
    }

    // register OEInstanceTag
    //if (!OEInstanceTag::registerPlugin())
    //{
    //    ERRLOG_RETURN_VALUE(FALSE, "Could not register OEInstanceTag plugin.");
    //}

    return TRUE;
}


/// Hook that is called during the shut down of CINEMA 4D. Here we can
/// deallocate all resources, that are not owned by CINEMA 4D.
void PluginEnd()
{}


/// Hook that is called for different messages.
Bool PluginMessage(LONG id,
                   void *data)
{
    return FALSE;
}
