#include "oe_obj_material.h"



OEOBJColorChannel::OEOBJColorChannel()
:
    mEnabled(false),
    mColor(0.0)
{}


bool OEOBJColorChannel::write(BufferedTextFile& file,
                              const char*       name) const
{
    if (mEnabled)
    {
        if (mFilename.Content())
        {
            return file.write("  map_") &&
                   file.write(name) &&
                   file.write(' ') &&
                   file.write(mFilename.GetString()) &&
                   file.write('\n');
        }
        else
        {
            return file.write("  ") &&
                   file.write(name) &&
                   file.write(' ') &&
                   file.write(mColor) &&
                   file.write('\n');
        }
    }
    return true;
}


OEOBJBumpChannel::OEOBJBumpChannel()
:
    mEnabled(false),
    mStrength(1.0)
{}


bool OEOBJBumpChannel::write(BufferedTextFile& file,
                             const char*       name) const
{
    if (mEnabled && mFilename.Content() && (mStrength != 0.0))
    {
        return file.write("  ") &&
               file.write(name) &&
               //file.write(" -bm ") &&
               //file.write(mStrength) &&
               file.write(' ') &&
               file.write(mFilename.GetString()) &&
               file.write('\n');
    }
    return true;
}


OEOBJFloatChannel::OEOBJFloatChannel()
:
    mValue(0.0)
{}


bool OEOBJFloatChannel::write(BufferedTextFile& file,
                              const char*       name) const
{
    if (mFilename.Content())
    {
        return file.write("  map_") &&
               file.write(name) &&
               file.write(' ') &&
               file.write(mFilename.GetString()) &&
               file.write('\n');
    } else {
        return file.write("  ") &&
               file.write(name) &&
               file.write(' ') &&
               file.write(mValue) &&
               file.write('\n');
    }
}


OEOBJMaterial::OEOBJMaterial()
:
    mNi(1.0),
    mTf(1.0),
    mHasTransparency(false)
{
    mKd.mEnabled = true;
    mKd.mColor.x = 0.9;
    mKd.mColor.y = 0.8;
    mKd.mColor.z = 0.6;
    mNs.mValue   = 0.0;
    mD.mValue    = 1.0;
}


bool OEOBJMaterial::write(BufferedTextFile& file,
                          const String&     name) const
{
    // determine illumination model
    LONG illumModel;
    if (mHasTransparency)
    {
        illumModel = 7;
    }
    else if (mKs.mEnabled)
    {
        illumModel = 2;
    }
    else
    {
        illumModel = 1;
    }

    // write the data
    return file.write("\nnewmtl ") && file.write(name) && file.write('\n') &&
           mKa.write(file, "Ka") &&
           mKd.write(file, "Kd") &&
           mKs.write(file, "Ks") &&
           mNs.write(file, "Ns") &&
           mD.write(file, "d") &&
           mBump.write(file, "bump") &&
           file.write("  Ni ") && file.write(mNi) && file.write('\n') &&
           file.write("  Tf ") && file.write(mTf) && file.write('\n') &&
           file.write("  illum ") && file.write(illumModel) &&
           file.write('\n');
}
