#include "c4d_symbols.h"
#include "oe_export.h"
#include "oe_preferences.h"



//-----------------------------------------------------------------------------
// OEPreferencesDlg


/// A little about dialog for the exporter.
class OEPreferencesDlg : public GeModalDialog
{
public:

    // --- Implementation(s) for GeModalDialog ---

    virtual Bool CreateLayout();

    virtual Bool InitValues();

    virtual Bool Command(
        LONG                id,
        const BaseContainer &msg);


private:

    void toggleGpu();

    void cleanupGPUString(
        const LONG id);
};


Bool OEPreferencesDlg::CreateLayout()
{
    return LoadDialogResource(OE_PREFERENCES_DLG, 0, 0);
}


Bool OEPreferencesDlg::InitValues()
{
    // get Octane path from global plugin settings
    BaseContainer* pluginData = GetWorldPluginData(PID_OE_PREFERENCES);
    if (pluginData)
    {
        SetFilename(IDD_PREFS_DLG_OCTANE_BINARY, pluginData, IDD_PREFS_DLG_OCTANE_BINARY);
        SetBool(IDD_PREFS_DLG_SPECIFY_GPU_DEV, pluginData, IDD_PREFS_DLG_SPECIFY_GPU_DEV);
        SetString(IDD_PREFS_DLG_GPU_DEV, pluginData, IDD_PREFS_DLG_GPU_DEV);
    }

    // toggle GPU device field
    toggleGpu();

    return TRUE;
}


Bool OEPreferencesDlg::Command(
    LONG                id,
    const BaseContainer &msg)
{
    switch (id)
    {
        // Octane binary chooser
        case IDB_PREFS_DLG_OCTANE_BINARY:
        {
            Filename octanePath;
            GetFilename(IDD_PREFS_DLG_OCTANE_BINARY, octanePath);
            if (fileSelect(octanePath,
                           FILESELECTTYPE_ANYTHING,
                           FILESELECT_LOAD,
                           GeLoadString(IDS_PREFS_DLG_OCTANE_BINARY_FS_TITLE)))
            {
                SetFilename(IDD_PREFS_DLG_OCTANE_BINARY, octanePath);
            }
            break;
        }

        // GPU settings
        case IDD_PREFS_DLG_SPECIFY_GPU_DEV:
            toggleGpu();
            break;
        case IDD_PREFS_DLG_GPU_DEV:
            cleanupGPUString(IDD_PREFS_DLG_GPU_DEV);
            break;

        // OK button: save settings and close the dialog
        case IDB_PREFS_DLG_OK:
        {
            BaseContainer pluginData;
            GetFilename(IDD_PREFS_DLG_OCTANE_BINARY, &pluginData, IDD_PREFS_DLG_OCTANE_BINARY);
            GetBool(IDD_PREFS_DLG_SPECIFY_GPU_DEV, &pluginData, IDD_PREFS_DLG_SPECIFY_GPU_DEV);
            GetString(IDD_PREFS_DLG_GPU_DEV, &pluginData, IDD_PREFS_DLG_GPU_DEV);
            SetWorldPluginData(PID_OE_PREFERENCES, pluginData, false);
            Close(TRUE);
            break;
        }

        // close the dialog without saving the settings
        case IDB_PREFS_DLG_CANCEL:
            Close(TRUE);
            break;

        default:
            break;
    }
    return TRUE;
}


void OEPreferencesDlg::toggleGpu()
{
    Bool specifyGpu;
    GetBool(IDD_PREFS_DLG_SPECIFY_GPU_DEV, specifyGpu);
    Enable(IDD_PREFS_DLG_GPU_DEV, specifyGpu);
}


void OEPreferencesDlg::cleanupGPUString(
    const LONG id)
{
    String entered, cleaned;
    GetString(id, entered);
    bool delim = false;
    for (LONG i=0; i<entered.GetLength(); ++i)
    {
        String::PChar ch = entered[i];
        if ((ch >= '0') && (ch <= '9'))
        {
            if (delim)
            {
                cleaned += " ";
                delim = false;
            }
            cleaned += String(1, ch);
        }
        else
        {
            delim = true;
        }
    }
    SetString(id, cleaned);
}



//-----------------------------------------------------------------------------
// Implementation of OEPreferences


OEPreferences *gPreferences = NULL;


bool OEPreferences::registerPlugin()
{
    // in the past the global preferences were stored under the export dialog ID
    // this got changed with version 0.13. if the container for the preferences
    // doesn't exist yet, create a new one and copy octane path from the previous
    // container.
    BaseContainer *pluginData = GetWorldPluginData(PID_OE_PREFERENCES);
    //if (!pluginData || !pluginData->GetIndexData(IDD_PREFS_DLG_OCTANE_BINARY))
    if (!pluginData || (pluginData->FindIndex(IDD_PREFS_DLG_OCTANE_BINARY) == NOTOK))
    {
        BaseContainer *oldPluginData = GetWorldPluginData(PID_OE_EXPORT);
        if (oldPluginData)
        {
            BaseContainer newPluginData;
            newPluginData.SetFilename(IDD_PREFS_DLG_OCTANE_BINARY,
                                      oldPluginData->GetString(_IDD_OE_DLG_OCTANE_BINARY));
            SetWorldPluginData(PID_OE_PREFERENCES, newPluginData, FALSE);
        }
    }

    // register the instance
    return RegisterCommandPlugin(PID_OE_PREFERENCES,
                                 GeLoadString(IDS_OE_PREFERENCES),
                                 0,
                                 NULL,
                                 GeLoadString(IDS_OE_PREFERENCES_DESCR),
                                 this);
}


Filename OEPreferences::octanePath()
{
    BaseContainer *pluginData = GetWorldPluginData(PID_OE_PREFERENCES);
    return (pluginData) ? pluginData->GetFilename(IDD_PREFS_DLG_OCTANE_BINARY) : Filename();
}


bool OEPreferences::hasSpecifiedGpus()
{
    BaseContainer *pluginData = GetWorldPluginData(PID_OE_PREFERENCES);
    return (pluginData) ? pluginData->GetBool(IDD_PREFS_DLG_SPECIFY_GPU_DEV) : false;
}


String OEPreferences::specifiedGpuStr()
{
    BaseContainer *pluginData = GetWorldPluginData(PID_OE_PREFERENCES);
    return (pluginData) ? pluginData->GetString(IDD_PREFS_DLG_GPU_DEV) : String();
}


Bool OEPreferences::Execute(BaseDocument* document)
{
    OEPreferencesDlg dialog;
    dialog.Open();
    return TRUE;
}
