#include "c4d_symbols.h"
#include "common.h"
#include "oe_scenehook.h"
#include "utilities.h"



/*****************************************************************************
 * Implementation of public member functions of class OESceneHook.
 *****************************************************************************/


NodeData *OESceneHook::alloc()
{
    return gNew OESceneHook;
}


bool OESceneHook::registerPlugin()
{
    return RegisterSceneHookPlugin(PID_OE_SCENEHOOK,
                                   "OESceneHook",
                                   0,
                                   alloc,
                                   EXECUTIONPRIORITY_EXPRESSION,
                                   0);
}


OESceneHook* OESceneHook::get(BaseDocument& document)
{
    // get scene hook node
    BaseList2D* pluginSceneHook = document.FindSceneHook(PID_OE_SCENEHOOK);
    if (!pluginSceneHook)
    {
        return 0;
    }

    // get scene hook data
    OESceneHook* sceneHook = (OESceneHook*)pluginSceneHook->GetNodeData();
    if (!sceneHook)
    {
        ERRLOG_RETURN_VALUE(0, "OESceneHook::get(): Could not obtain scenehook from active document.");
    }

    // do some conversion work for backwards compatibility
    sceneHook->convertRenderTargetSettings();

    return sceneHook;
}


Bool OESceneHook::Read(GeListNode* node,
                       HyperFile*  hf,
                       LONG        level)
{
    hf->ReadContainer(&mExportDlgSettings, TRUE);
    return TRUE;
}


Bool OESceneHook::Write(GeListNode* node,
                        HyperFile*  hf)
{
    hf->WriteContainer(mExportDlgSettings);
    return TRUE;
}


Bool OESceneHook::CopyTo(NodeData*   dest,
                         GeListNode* snode,
                         GeListNode* dnode,
                         COPYFLAGS   flags,
                         AliasTrans* trn)
{
    static_cast<OESceneHook*>(dest)->mExportDlgSettings = mExportDlgSettings;
    return TRUE;
}


inline void OESceneHook::copyExportSetting(
    BaseContainer &destContainer,
    const LONG    id)
{
    destContainer.SetData(id, mExportDlgSettings.GetData(id));
}


void OESceneHook::convertRenderTargetSettings()
{
    // return early if the render target settings already exist
    if (mExportDlgSettings.FindIndex(IDD_OE_DLG_RENDERTARGET_SETTINGS) != NOTOK)
    {
        return;
    }

    // get render target list and return early if there are none
    const BaseContainer *renderTargets =
        mExportDlgSettings.GetContainerInstance(IDD_OE_DLG_RENDERTARGET_LIST);
    if (!renderTargets)
    {
        return;
    }

    // in the past there was no concept of render target settings. the export
    // configuration got applied to all render targets. now we have one export
    // configuration per render target and create them all based on the original
    // export configuration.

    // copy render target settings from export dialog settings into new container
    BaseContainer config;
    copyExportSetting(config, IDD_OE_DLG_EXPORT_RESOLUTION);
    copyExportSetting(config, IDD_OE_DLG_RES_WIDTH);
    copyExportSetting(config, IDD_OE_DLG_RES_HEIGHT);
    copyExportSetting(config, IDD_OE_DLG_RES_SCALE);
    copyExportSetting(config, IDD_OE_DLG_EXPORT_CAMERA);
    copyExportSetting(config, IDD_OE_DLG_ACTIVE_CAMERA);
    copyExportSetting(config, IDD_OE_DLG_EXPORT_LENS_APERTURE);
    copyExportSetting(config, IDD_OE_DLG_LENS_APERTURE);
    copyExportSetting(config, IDD_OE_DLG_EXPORT_FOCAL_DEPTH);
    copyExportSetting(config, IDD_OE_DLG_FOCUS_OF_CAM_OBJECT);
    copyExportSetting(config, IDD_OE_DLG_FOCAL_DEPTH);
    copyExportSetting(config, IDD_OE_DLG_EXPORT_MOTION_BLUR);
    copyExportSetting(config, IDD_OE_DLG_MOTION_BLUR);
    copyExportSetting(config, IDD_OE_DLG_EXPORT_SUN_DIR);
    copyExportSetting(config, IDD_OE_DLG_SUN_LIGHT);

    // loop over render targets and copy export configuration into render target settings
    BaseContainer renderTargetSettings;
    for (LONG id=0; (id == 0) || renderTargets->GetString(id).Content(); ++id)
    {
        renderTargetSettings.SetContainer(id, config);
    }

    // store render target settings in export settings
    mExportDlgSettings.SetContainer(IDD_OE_DLG_RENDERTARGET_SETTINGS, renderTargetSettings);
}
