#include <unistd.h>
#include <Carbon/Carbon.h>
#include <CoreFoundation/CoreFoundation.h>



ProcessSerialNumber _runProgramInternal(const char* appPath,
                                        char *const args[],
                                        const bool  runInBackground)
{
    ProcessSerialNumber processSN = {0, 0};

    pid_t processID = fork();
    switch (processID)
	{
        // fork() failed
        case -1:
            return processSN;
        // child process
        case 0:
            execv(appPath, args);
            exit(EXIT_FAILURE);
            break;
        // parent proces
        default:
            break;
    }

    // get PSN from process ID and if that fails, try again up to 20x waiting
    // up to 10s in total
    if (GetProcessForPID(processID, &processSN) != noErr)
    {
        // loop and wait for up to 10s to get the process serial number
        for (int c=0; c<20; ++c)
        {
            usleep(500000);
            if (GetProcessForPID(processID, &processSN) == noErr)
            {
                break;
            }
            else
            {
                processSN.highLongOfPSN = 0;
                processSN.lowLongOfPSN  = 0;
            }
        }
        // just to be sure, we kill the child process to avoid zombies
        if (!processSN.highLongOfPSN && !processSN.lowLongOfPSN)
        {
            kill(processID, SIGTERM);
        }
    }

    // give focus to new process and return
    if (!runInBackground) { SetFrontProcess(&processSN); }
    return processSN;
}


bool _isValidInternal(const ProcessSerialNumber& process)
{
    return process.highLongOfPSN || process.lowLongOfPSN;
}


bool _programIsRunningInternal(const ProcessSerialNumber& process)
{
    ProcessInfoRec info;
    memset(&info, 0, sizeof(info));
    OSErr errCode = GetProcessInformation(&process, &info);
    return (errCode >= 0);
}


void _stopProgramInternal(const ProcessSerialNumber& process)
{
    KillProcess(&process);
}


void _convertToPosixPathInternal(char*        hfsPathUTF8,
                                 const size_t maxLen)
{
    // convert plain C string to CoreFoundation CFString
    CFStringRef pathCFString = CFStringCreateWithCString(NULL,
                                                         hfsPathUTF8,
                                                         kCFStringEncodingUTF8);

    // convert CFString to CFURL and release CFString
    CFURLRef url = CFURLCreateWithFileSystemPath(NULL,
                                                 pathCFString,
                                                 kCFURLHFSPathStyle,
                                                 FALSE);
    CFRelease(pathCFString);

    // convert CFURL back to a CFString using the POSIX path style and release CFURL
    pathCFString = CFURLCopyFileSystemPath(url, kCFURLPOSIXPathStyle);
    CFRelease(url);

    // convert CFString back to plain C string and release CFString
    CFStringGetCString(pathCFString,
                       hfsPathUTF8,
                       maxLen,
                       kCFStringEncodingUTF8);
    CFRelease(pathCFString);
    hfsPathUTF8[maxLen-1] = '\0';
}


void _convertToHSFPathInternal(char*        posixPathUTF8,
                               const size_t maxLen)
{
    // convert plain C string to CoreFoundation CFString
    CFStringRef pathCFString = CFStringCreateWithCString(NULL,
                                                         posixPathUTF8,
                                                         kCFStringEncodingUTF8);

    // convert CFString to CFURL and release CFString
    CFURLRef url = CFURLCreateWithFileSystemPath(NULL,
                                                 pathCFString,
                                                 kCFURLPOSIXPathStyle,
                                                 FALSE);
    CFRelease(pathCFString);

    // convert CFURL back to a CFString using the POSIX path style and release CFURL
    pathCFString = CFURLCopyFileSystemPath(url, kCFURLHFSPathStyle);
    CFRelease(url);

    // convert CFString back to plain C string and release CFString
    CFStringGetCString(pathCFString,
                       posixPathUTF8,
                       maxLen,
                       kCFStringEncodingUTF8);
    CFRelease(pathCFString);
    posixPathUTF8[maxLen-1] = '\0';
}
