#ifndef STL_CONTAINERS_H
#define STL_CONTAINERS_H 1



#include <list>
#include <map>
#include <set>
#include <vector>
#include "c4d_allocator.h"



template <typename T>
class StdList : public std::list<T,C4DAllocator<T> >
{
    typedef std::list<T,C4DAllocator<T> >  SuperT;

public:

    inline StdList()
    {}

    explicit inline StdList(typename SuperT::size_type n,
                            const T&                   value = T())
    :
        SuperT(n, value)
    {}

    template <typename InputIterator>
    inline StdList(const InputIterator& first,
                   const InputIterator& last)
    :
        SuperT(first, last)
    {}
};



template <typename Key,
          typename T,
          typename Compare = std::less<Key> >
class StdMap : public std::map<Key,T,Compare,C4DAllocator<std::pair<const Key,T> > >
{
    typedef std::map<Key,T,Compare,C4DAllocator<std::pair<const Key,T> > >  SuperT;

public:

    explicit inline StdMap(const Compare& compare = Compare())
    :
        SuperT(compare)
    {}

    template <typename InputIterator>
    inline StdMap(const InputIterator& first,
                  const InputIterator& last,
                  const Compare&       compare = Compare())
    :
        SuperT(first, last, compare)
    {}
};



template <typename Key,
          typename Compare = std::less<Key> >
class StdSet : public std::set<Key,Compare,C4DAllocator<Key> >
{
    typedef std::set<Key,Compare,C4DAllocator<Key> >  SuperT;

public:

    explicit inline StdSet(const Compare& compare = Compare())
    :
        SuperT(compare)
    {}

    template <typename InputIterator>
    inline StdSet(const InputIterator& first,
                  const InputIterator& last,
                  const Compare&       compare = Compare())
    :
        SuperT(first, last, compare)
    {}
};



template <typename T>
class StdVector : public std::vector<T,C4DAllocator<T> >
{
    typedef std::vector<T,C4DAllocator<T> > SuperT;

public:

    inline StdVector()
    {}

    explicit inline StdVector(typename SuperT::size_type n,
                              const T&                   value = T())
    :
        SuperT(n, value)
    {}

    template <typename InputIterator>
    inline StdVector(const InputIterator& first,
                     const InputIterator& last)
    :
        SuperT(first, last)
    {}
};



#endif  // #ifndef STL_CONTAINERS_H
